/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "XmlObject.h"

#include <QFile>
#include <QTextStream>
#include <algorithm>

XmlObject::XmlObject(const QString& objectName) { setObjectName(objectName); }

XmlObject::XmlObject(const XmlObject& other) { copy(other); }

bool XmlObject::loadFromFile(const QString& filePath)
{
    bool result = false;
    QFile file(filePath);
    if (file.exists() && file.open(QIODevice::ReadOnly))
    {
        QString msgDefFileContent = QTextStream(&file).readAll();
        if (msgDefFileContent != "")
        {
            this->fromXml(msgDefFileContent);
            result = true;
        }
        file.close();
    }
    return result;
}

bool XmlObject::writeToFile(const QString& filePath, const QString& codec) const
{
    bool result = false;
    QFile file(filePath);
    if (file.open(QIODevice::WriteOnly))
    {
        QTextStream textStream(&file);
        textStream.setCodec(codec.toUtf8());
        textStream << this->toXml();
        file.close();
        result = true;
    }
    return result;
}

QString XmlObject::toXml() const
{
    QString output;
    QXmlStreamWriter xWriter(&output);
    xWriter.setAutoFormatting(true);
    toXml(&xWriter);
    return output;
}

void XmlObject::toXml(QXmlStreamWriter* xmlWriter) const
{
    xmlWriter->writeStartElement(xmlObjectName);

    for (QString key : attributes.keys())
    {
        xmlWriter->writeAttribute(key, attributes[key]);
    }

    if (getTextValue() != "" && childCount() == 0)
    {
        xmlWriter->writeCharacters(getTextValue());
    }

    for (XmlObject c : childList)
    {
        c.toXml(xmlWriter);
    }

    xmlWriter->writeEndElement();
}

void XmlObject::fromXml(const QString& txt)
{
    QXmlStreamReader xReader(txt);
    xReader.readNextStartElement();
    fromXml(&xReader);
}

void XmlObject::fromXml(QXmlStreamReader* xmlReader)
{
    childList.clear();
    attributes.clear();
    if (xmlReader->isStartElement())
    {
        setObjectName(xmlReader->name().toString());
        for (auto attr : xmlReader->attributes())
        {
            attributes[attr.name().toString()] = attr.value().toString();
        }
    }

    while (!xmlReader->atEnd())
    {
        xmlReader->readNext();

        if (xmlReader->isCharacters() && !xmlReader->isWhitespace())
        {
            setTextValue(xmlReader->text().toString());
        }
        else if (xmlReader->isStartElement())
        {
            XmlObject c;
            c.fromXml(xmlReader);
            childList.append(c);
        }
        else if (xmlReader->isEndElement())
        {
            return;
        }
    }
}

void XmlObject::copy(const XmlObject& other)
{
    childList.clear();
    attributes.clear();
    setObjectName(other.getObjectName());
    setTextValue(other.getTextValue());
    for (QString attr : other.attributes.keys())
    {
        attributes[attr] = other.attributes[attr];
    }
    for (int i = 0; i < other.childCount(); i++)
    {
        addChild(other.childAt(i));
    }
}

QString XmlObject::getTextValue() const { return textValue; }

void XmlObject::setTextValue(const QString& value) { textValue = value; }

QList<XmlObject*> XmlObject::deepSearchObjects(const QString& objectName)
{
    auto pred = [objectName](const XmlObject* obj)
    { return obj->getObjectName() == objectName; };
    return deepSearchObjects(pred);
}

QList<XmlObject*> XmlObject::deepSearchObjects(
    std::function<bool(const XmlObject* obj)> predicate)
{
    QList<XmlObject*> results;

    if (predicate(this))
    {
        results.append(this);
    }

    for (int i = 0; i < childCount(); i++)
    {
        XmlObject* child = getChild(i);
        results.append(child->deepSearchObjects(predicate));
    }

    return results;
}

void XmlObject::operator=(const XmlObject& other) { copy(other); }

void XmlObject::removeChild(int i)
{
    if (i < childCount())
        childList.removeAt(i);
}

void XmlObject::swapChildAt(int i, int j)
{
    if (i > 0 && j > 0 && i < childCount() && j < childCount())
    {
        childList.swapItemsAt(i, j);
    }
}

QString XmlObject::getChildObjectValue(const QString& objectName)
{
    auto result = std::find_if(childList.begin(), childList.end(),
                               [&](XmlObject child)
                               { return child.getObjectName() == objectName; });

    if (result != childList.end())
        return result->getTextValue();
    else
        return "";
}

void XmlObject::clearAttributes() { attributes.clear(); }

void XmlObject::clearChilds() { childList.clear(); }

void XmlObject::reset()
{
    setObjectName("");
    clearChilds();
    clearAttributes();
}

bool XmlObject::isEmpty()
{
    return getObjectName() == "" && childList.isEmpty() &&
           getTextValue() == "" && attributes.isEmpty();
}

QString XmlObject::getAttribute(const QString& name,
                                const QString& defaultValue) const
{
    if (attributes.contains(name))
    {
        return attributes[name];
    }

    return defaultValue;
}

bool XmlObject::hasAttribute(const QString& name) const
{
    return attributes.contains(name);
}

int XmlObject::attributeCount() const { return attributes.count(); }

QMapIterator<QString, QString> XmlObject::attributesIterator() const
{
    return QMapIterator<QString, QString>(attributes);
}

bool XmlObject::getAttribute(const QString& name, int& value) const
{
    QString val = getAttribute(name);
    bool ok;
    if (val != "")
    {
        int temp = val.toInt(&ok);
        if (ok)
        {
            value = temp;
            return true;
        }
    }
    return false;
}

bool XmlObject::getAttribute(const QString& name, float& value) const
{
    QString val = getAttribute(name);
    bool ok;
    if (val != "")
    {
        float temp = val.toFloat(&ok);
        if (ok)
        {
            value = temp;
            return true;
        }
    }
    return false;
}

void XmlObject::addAttribute(const QString& name, const QString& value)
{
    attributes[name] = value;
}

void XmlObject::addAttribute(const QString& name, int value)
{
    addAttribute(name, QString::number(value));
}

void XmlObject::addAttribute(const QString& name, float value)
{
    addAttribute(name, QString::number(value));
}

int XmlObject::addChild(const XmlObject& c)
{
    childList.append(c);
    return childList.count() - 1;
}

void XmlObject::addChilds(const QList<XmlObject>& childs)
{
    childList.append(childs);
}

QString XmlObject::getObjectName() const { return xmlObjectName; }

void XmlObject::setObjectName(const QString& value) { xmlObjectName = value; }

int XmlObject::childCount() const { return childList.count(); }

XmlObject XmlObject::childAt(int index) const { return childList.at(index); }

int XmlObject::searchChild(const QString& name) const
{
    for (int i = 0; i < childCount(); i++)
        if (childList[i].getObjectName() == name)
            return i;

    return -1;
}

XmlObject* XmlObject::getChild(int index) { return &childList[index]; }

QMap<QString, QString> XmlObject::getAttributes() const { return attributes; }