/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "IncomingMessagesViewerModule.h"

#include <QHBoxLayout>

#include "Components/SubscriptionsPanel/SubscriptionsPanel.h"

IncomingMessagesViewerModule::IncomingMessagesViewerModule()
    : Module(ModuleId::INCOMING_MESSAGES_VIEWER)
{
    setupUi();
    customContextMenuActionSetup();
}

IncomingMessagesViewerModule::~IncomingMessagesViewerModule()
{
    for (auto filter : filters)
        MessageBroker::getInstance().unsubscribe(filter, this);
}

XmlObject IncomingMessagesViewerModule::toXmlObject()
{
    XmlObject obj = Module::toXmlObject();

    obj.addAttribute("keepOnlyLastMessage", keepOnlyLastMessage ? "1" : "0");
    obj.addAttribute("useTimestamp", useTimestamp ? "1" : "0");

    for (int i = 0; i < filters.count(); i++)
    {
        XmlObject subscription("subscription");
        subscription.addAttribute("filter", filters[i].toString());
        obj.addChild(subscription);
    }

    return obj;
}

void IncomingMessagesViewerModule::fromXmlObject(const XmlObject& xmlObject)
{
    keepOnlyLastMessage = xmlObject.getAttribute("keepOnlyLastMessage") == "1";
    keepOnlyLastMessageAction->setChecked(keepOnlyLastMessage);
    useTimestamp = xmlObject.getAttribute("useTimestamp") == "1";
    useTimestampAction->setChecked(useTimestamp);

    for (int i = 0; i < xmlObject.childCount(); i++)
    {
        XmlObject child = xmlObject.childAt(i);
        if (child.getObjectName() == "subscription")
            onFilterAdded(Filter::fromString(child.getAttribute("filter")));
    }
}

void IncomingMessagesViewerModule::onSubscribeClicked()
{
    SubscriptionsPanel* panel = new SubscriptionsPanel(filters);
    panel->setWindowTitle("Graph subscriptions");
    connect(panel, &SubscriptionsPanel::filterAdded, this,
            &IncomingMessagesViewerModule::onFilterAdded);
    connect(panel, &SubscriptionsPanel::filterRemoved, this,
            &IncomingMessagesViewerModule::onFilterRemoved);
    panel->show();
}

void IncomingMessagesViewerModule::onFilterAdded(const Filter& filter)
{
    filters.append(filter);

    MessageBroker::getInstance().subscribe(
        filter, this,
        [&](const Message& message, const Filter& filter)
        {
            QString oldText = "";
            QString time    = "";

            // Save the old text
            if (!keepOnlyLastMessage)
                oldText = edit->toPlainText().mid(0, 10000);

            if (useTimestamp)
            {
                // If available  show the timestamp
                auto timestamp = message.getField("timestamp");
                time += "[";
                time += QString::number(
                    (float)timestamp.getUnsignedInteger() / 1e6, 'f', 1);
                time += "] ";
            }
            else
            {
                time += "[" + QTime::currentTime().toString("hh.mm.ss") + "] ";
            }

            // Updated the content
            edit->setText(time + message.toString() + "\n" + oldText);
        });
}

void IncomingMessagesViewerModule::onFilterRemoved(const Filter& filter)
{
    filters.removeAll(filter);
    MessageBroker::getInstance().unsubscribe(filter, this);
}

void IncomingMessagesViewerModule::setupUi()
{
    edit = new QTextEdit();
    edit->setReadOnly(true);
    edit->setContextMenuPolicy(Qt::ContextMenuPolicy::NoContextMenu);

    QHBoxLayout* layout = new QHBoxLayout();
    layout->setContentsMargins(0, 0, 0, 0);
    layout->addWidget(edit);
    setLayout(layout);
}

void IncomingMessagesViewerModule::customContextMenuActionSetup()
{
    QAction* clear = new QAction("Clear");
    connect(clear, &QAction::triggered, this, [this]() { edit->clear(); });
    customContextMenuActions.append(clear);

    QAction* subscriptions = new QAction("Manage subscriptions");
    connect(subscriptions, &QAction::triggered, this,
            &IncomingMessagesViewerModule::onSubscribeClicked);
    customContextMenuActions.append(subscriptions);

    keepOnlyLastMessageAction = new QAction("Keep only last message");
    keepOnlyLastMessageAction->setCheckable(true);
    connect(keepOnlyLastMessageAction, &QAction::toggled, this,
            [=](bool value) { keepOnlyLastMessage = value; });
    customContextMenuActions.append(keepOnlyLastMessageAction);

    useTimestampAction = new QAction("Use timestamp");
    useTimestampAction->setCheckable(true);
    connect(useTimestampAction, &QAction::toggled, this,
            [=](bool value) { useTimestamp = value; });
    customContextMenuActions.append(useTimestampAction);
}
