/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "ValvesViewer.h"

#include <Components/FilterSelector/FilterSelector.h>
#include <Core/MessageBroker/MessageBroker.h>

#include <iostream>

ValvesViewer::ValvesViewer() : Module(ModuleId::VALVES_VIEWER)
{
    setupUi();
    customContextMenuActionSetup();
}

ValvesViewer::~ValvesViewer()
{
    MessageBroker::getInstance().unsubscribe(filter, this);
}

XmlObject ValvesViewer::toXmlObject()
{
    XmlObject obj = Module::toXmlObject();

    obj.addAttribute("filter", filter.toString());

    return obj;
}

void ValvesViewer::fromXmlObject(const XmlObject& xmlObject)
{
    auto filter = Filter::fromString(xmlObject.getAttribute("filter"));
    setFilter(filter);
}

void ValvesViewer::setupUi()
{
    outerLayout = new QHBoxLayout;
    outerLayout->setContentsMargins(0, 0, 0, 0);
    outerLayout->setSpacing(0);

    for (auto labelText : ValvesList::valvesLabels)
    {
        QLabel* label = new QLabel;
        label->setText(labelText);
        label->setAlignment(Qt::AlignCenter);
        label->setContentsMargins(4, 4, 4, 4);
        QString baseStyle{baseStylesheet};
        label->setStyleSheet(baseStyle);
        outerLayout->addWidget(label);
    }

    setLayout(outerLayout);
}

void ValvesViewer::customContextMenuActionSetup()
{
    QAction* action = new QAction("Choose topic and field");
    connect(action, &QAction::triggered, this,
            &ValvesViewer::onConfigureClicked);

    customContextMenuActions.append(action);
}

void ValvesViewer::onConfigureClicked()
{
    FilterSelector::selectFilter(
        filter, [this](const Filter& newFilter) { setFilter(newFilter); });
}

void ValvesViewer::setFilter(const Filter& newFilter)
{
    MessageBroker::getInstance().unsubscribe(filter, this);
    MessageBroker::getInstance().subscribe(
        newFilter, this,
        [this](const Message& message, const Filter& filter)
        { onMsgReceived(message); });
    filter = newFilter;
}

void ValvesViewer::onMsgReceived(const Message& msg)
{
    // Couple the valves with their own state
    QMap<ValvesList::Valve, bool> map{
        {ValvesList::Valve::N2O_FILLING,
         msg.getField("n2o_filling_valve_state").getUnsignedInteger()},
        {ValvesList::Valve::N2O_RELEASE,
         msg.getField("n2o_release_valve_state").getUnsignedInteger()},
        {ValvesList::Valve::N2O_DETACH,
         msg.getField("n2o_detach_state").getUnsignedInteger()},
        {ValvesList::Valve::N2O_VENTING,
         msg.getField("n2o_venting_valve_state").getUnsignedInteger()},
        {ValvesList::Valve::N2_FILLING,
         msg.getField("n2_filling_valve_state").getUnsignedInteger()},
        {ValvesList::Valve::N2_RELEASE,
         msg.getField("n2_release_valve_state").getUnsignedInteger()},
        {ValvesList::Valve::N2_DETACH,
         msg.getField("n2_detach_state").getUnsignedInteger()},
        {ValvesList::Valve::N2_QUENCHING,
         msg.getField("n2_quenching_valve_state").getUnsignedInteger()},
        {ValvesList::Valve::N2_3WAY,
         msg.getField("n2_3way_valve_state").getUnsignedInteger()},
        {ValvesList::Valve::MAIN,
         msg.getField("main_valve_state").getUnsignedInteger()},
        {ValvesList::Valve::NITROGEN,
         msg.getField("nitrogen_valve_state").getUnsignedInteger()},
        {ValvesList::Valve::IGNITION,
         msg.getField("ignition_state").getUnsignedInteger()},
        {ValvesList::Valve::TARS,
         msg.getField("tars_state").getUnsignedInteger()}};

    // Define the "active" and the "not active" look
    QString baseStyle{baseStylesheet};
    QString completedStyle = "background-color:green;" + baseStyle;
    QString errorStyle     = "background-color:red;" + baseStyle;

    // Iterate over the valves to assign them the correct look
    for (auto i = map.cbegin(), end = map.cend(); i != end; i++)
    {
        if (i.value())
        {
            // The valve is opened
            auto label =
                outerLayout->itemAt(static_cast<int>(i.key()))->widget();
            label->setStyleSheet(completedStyle);
        }
        else
        {
            // The valve is closed
            auto label =
                outerLayout->itemAt(static_cast<int>(i.key()))->widget();
            label->setStyleSheet(errorStyle);
        }
    }
}
